#!/usr/bin/perl

# ourmon 2.6 configure.pl
#
# syntax:
# 2 ways to install:
#
# 1. # ./configure.pl
# 	installs in current working directory.  NOTE: we assume
#	that this is at the top of the ourmon development tree (src is in ./src).
# 2. # ./configure.pl /dirpath
#	builds ourmon locally (src is still ./src), but makes bin/scripts so
#	that entire built tree can be installed somewhere else via a cp -r.
#	NOTE!!!.  ./mrourmon is appended to the /dirpath, so if you want 
#		/usr/local/mrourmon supply /usr/local
#
# assumptions:
# 	We assume that the dependencies have been installed.
#
#	1. a good libpcap
#		this is for src/ourmon/ourmon
#	2. rrdtool in such a way that the libs are in the perl path
#		this is for src/web.code/omupdate.pl.  NOTE: not getting
#		this right is a common problem.  see the INSTALL notes.
#
# ourmon install/configure script 
#
# the script will install the front-end
# and/or the graphics back-end as two different parts.
#
# Either front-end or back-end can be installed
# separately or together on the same system or two different systems.
#
# All parts of ourmon are installed in the ourmon bin.
#
# we build the following directory structure
#
# bin logs rrddata tmp 
#
# we assume we have a local src and etc
#
# we put in a symlink for the web directory.  The user
# should know where it is before running this script.
#
# Only one outside world file may be touched (not the default) 
# and that is the root crontab.
# If vixie, we assume /etc/crontab.
# If not vixie, we assume /var/spool/cron/crontab/root
# --------------------------------------------------------------------
# question protocol works as follows:
#
# 1. set default in variable
#$YES_CORRECT="y";	
# 2. ask question, mention default
# print "do you want to install /etc/ourmon.conf? [y] ";
# $line = <STDIN>;
# if line is empty, we use default
# if ($line =~ /^\n/) { 
# }
# else {
# 	chomp($YES_CORRECT = $line);  
# }
# lc reduces to lower-case in case answer was Y
# if (lc($YES_CORRECT) eq "y") {
#	print "$YES_CORRECT is y\n";
#}
# else {
# print "$YES_CORRECT is not y\n";
# }
#
######################################################################
$netset = "n";

use File::Basename();
use Cwd;

# find our cwd and systype
my $cwd = getcwd();

# figure out systype
$systype = `uname`;
chomp($systype);

$reltype = `uname -r`;
chomp($reltype);

# INSTALL_PREFIX is used to say "install here", however this really really
# means "build here".  If we don't move the result, then fine.

# FINAL_SCRIPT_DIR is where the final install ends up EVENTUALLY
# and where scripts have to reference.  This distinction allows
# ourmon to be built locally and installed somewhere else.

# assume the same (no args)
$INSTALL_PREFIX = $FINAL_SCRIPT_DIR = $cwd;
if ($ARGV[0]) {
	$FINAL_SCRIPT_DIR = $ARGV[0] . "/mrourmon";
}

########################################################
# Variables
########################################################
#
$FRONTEND="y";
$BACKEND="y";
$LOGGING="on";		# don't turn it off
$interface="eth0";
$ourmon_script = "ourmon.sh";
$omupdate_script = "omupdate.sh";
$netmask = "127.0.0.1/32";

# default absolute web paths, may be wrong
$webpath = "/usr/local/apache/htdocs/ourmon";
if ($systype eq "FreeBSD") {
	$webpath = "/usr/local/www/data/ourmon";
	# guess the bsd interface whilst we are at it
	$interface = guess_bsd_if();
	$fbsd6 = 0;
	if ($reltype =~ /6\./) {
		$fbsd6 = 1;
	}
}

# this is based on path for ubuntu apache2 package
if ($systype eq "Linux") {
	$webpath = "/var/www/apache2-default/ourmon";
}

$crontab_file = "/tmp/crontab.ourmon.xxx";
$wgetpath = "/usr/local/bin/wget";

$flowsize = 10000000;
$udpweight = 10000000;
$batchip_file = "batchip.sh";
$batchipall_file = "batchipall.sh";
$topnip = "";
$topnudp = "";
$topntcp = "";
$topnalert = "";
$syslog_conf_file = "scripts/syslog.conf";
$nsyslog_conf_file = "scripts/newsyslog.conf";
$drawtopn_ok = 0;
$ourmon_ok = 0;
$rc;

$ourpath = $FINAL_SCRIPT_DIR;
chomp($ourpath);

# make sure certain directories exist
if (!-d "bin") {
	system("mkdir bin");
}
if (!-d "tmp") {
	system("mkdir tmp");
}
if (!-d "logs") {
	system("mkdir logs");
}
if (!-d "rrddata") {
	system("mkdir rrddata");
}
$ourbin = $ourpath . "/bin";
$rrdbase = $ourpath . "/rrddata";
$logpath = $ourpath . "/logs";
$tmppath = $ourpath . "./tmp";
$ourmon_conf = $ourpath . "/etc/ourmon.conf";
# supplied to ourmon front-end as param
$monlite = $ourpath . "/tmp";
# must rewrite etc/ourmon.conf for these
$tcpworm = $ourpath . "/tmp/tcpworm.txt";
$p2pfile = $ourpath . "/tmp/p2pfile.txt";
$ircfile = $ourpath . "/tmp/irc.txt";
my $olibs="LIBS=";

print "configuration script to install ourmon.\n";
print "note: default is suggested like so: [default]\n";
print "note: just hit carriage-return for default actions\n";
print "---------------------------------------------------\n";

print "Would you like to install the ourmon probe? [y] ";
$line = <STDIN>;
unless ($line =~ /^\n/) {
  chomp($FRONTEND = $line);
}

# install ourmon probe
if (lc($FRONTEND) eq "y") {
	print "Front-end configuration phase started ####################\n";

	# if BSD but not linux try and choose local/lib over /lib if possible
	# this effects the libs we pass in for the ourmon compile and makefile
	if ($systype eq "FreeBSD") {
		# find out where libpcre.a is -- prefer /usr/local/lib 
		# but /usr/lib or /lib/libpcre.a is ok
		if (-e "/usr/local/lib/libpcre.a") {
			$olibs = $olibs . "/usr/local/lib/libpcre.a "; 
		}
		else {
			if (-e "/usr/lib/libpcre.a") {
				$olibs = $olibs . "/usr/lib/libpcre.a "; 
			}
			else {
				if (-e "/lib/libpcre.a") {
					$olibs = $olibs . "/lib/libpcre.a "; 
				}
				else {
					print "did not find libpcre.a anywhere\n"; 
				}
			}
		}

		# find out if libpcap is in /usr/local or /usr/lib
		# go with /usr/local if found
		# tell user about it ...
		my $localpcap = 0;
		if (-e "/usr/local/lib/libpcap.a") {
			$olibs = $olibs . "/usr/local/lib/libpcap.a"; 
			$localpcap = 1;
		}
		else {
			if (-e "/usr/lib/libpcap.a") {
				$olibs = $olibs . "/usr/lib/libpcap.a"; 
				$localpcap = 0;
			}
			else {
				print "did not find libpcap.a in either /usr/local/lib or /usr/lib\n"; 
			}
		}

		print "pcap in general needs to be reinstalled from www.tcpdump.org before ourmon install\n";
		if ($localpcap == 1) {
			print "found pcap lib in /usr/local/lib/libpcap.a - which is a good thing\n";
		}
		else {
			print "did NOT find pcap lib in /usr/local/lib/libpcap.a\n";
			print "if ourmon fails at boot due to claimed syntax errors in $ourmon_conf\n";
			print "OR if on FreeBSD only ourmon fails to properly size BPF buffer (see INSTALL)\n";
			print "please download and reinstall libpcap and start the configure over again\n";
		} 
		print "hit <CR> to continue: ";
		$line = <STDIN>;
	}

	#1. compile/install ourmon 
	$YES_CORRECT="y";
	print "\nWould you like to compile/install ourmon? [y] ";
	$line = <STDIN>;
	unless ($line =~ /^\n/) {
	  chomp($YES_CORRECT = $line);  
	}
	if (lc($YES_CORRECT) eq "y") {
		chdir("src/ourmon");
		# pass in libs whereever found
		if ($systype eq "FreeBSD") {
			print "ourmon build: using make -f Makefile.bsd $olibs\n";
			$rc = system("make -f Makefile.bsd \"$olibs\"");
		}
		if ($systype eq "Linux") {
			print "ourmon build: using make -f Makefile.linux\n";
			$rc = system("make -f Makefile.linux");
		}
		if ($rc == 0) {
			system("cp ourmon ../../bin");
			$ourmon_ok = 1;
		}
		else {
			print "ourmon compile failed\n";
			print "See INSTALL file for more info.\n";
			print "you may need to get a new libpcap library\n"; 
			print "or try making src/ourmon/ourmon by hand.\n";
			print "config terminated.\n";
			exit (1);
		}
		chdir("../..");
		system("chmod +x ./bin/ourmon");
	}
	#2 notify user of intent to use ourmon.conf as ourmon.conf file
	print "\nNext we determine the ourmon config/filter file to use.\n";
	print "By default, we use the local $FINAL_SCRIPT_DIR/etc/ourmon.conf to provide input filters to ourmon.\n";
	print "WARNING: you should read/edit/understand ourmon.conf!\n";
	$YES_CORRECT="n";
	print "Do you want to use another ourmon.conf file in some other directory than $FINAL_SCRIPT_DIR/etc? [n] ";
	$line = <STDIN>;
	unless ($line =~ /^\n/) {
	  	chomp($YES_CORRECT = $line);  
	}
	if (lc($YES_CORRECT) eq "y") {
		print "enter the complete filename for the ourmon.conf file. [mumble/ourmon.conf] ";
		$line = <STDIN>;
		chomp($line);
		$ourmon_conf = $line;
		print "ourmon.conf file to be used: <$ourmon_conf>\n";
	}
	print "\nNext we suggest one modification to the ourmon.conf file.\n";
	print "\nIf this is a default install, you should change the following config directive:\n"; 
	print "\n\ttopn_syn_homeip network/netmask\n";
	print "\nand set it to your home network and mask (A.B.C.D/maskbits style)\n";
	print "Do you want to change the topn_syn home network address? [y] ";
	$YES_CORRECT="y";
	$line = <STDIN>;
	unless ($line =~ /^\n/) {
	  	chomp($YES_CORRECT = $line);  
	}
	if (lc($YES_CORRECT) eq "y") {
		print "note: the home net address may be a subnet or host address (/32).\n";
		print "enter a home net address and mask. [127.0.0.1/32] ";
		$line = <STDIN>;
		if ($line =~ /^\n/) { 
			$line = $netmask;
		}
		else {
			chomp($line);
			$netmask = $line;
		}
#print "DEBUG: (netmask=$netmask) ($line)\n";
		# note this sed pattern will replace any valid ip/mask pair
#		my $oldline="topn_syn_homeip[ \t]*[0-9,.]*[ \t]*[0-9,.]*";
 		my $oldline="topn_syn_homeip[ \t]*[0-9,.\/]*";
		sed_line_munge($ourmon_conf, $oldline, "topn_syn_homeip $line");
		print "netmask: $netmask\n";
		$netset = "y";
	}

	# make sure everything in bin is executable
	system("chmod +x bin/*");

	#3 create startup script for ourmon
	$YES_CORRECT="y";
	print "\nDo you want to install the ourmon startup script in the ourmon bin? [y] ";
	$line = <STDIN>;
	unless ($line =~ /^\n/) {
	  	chomp($YES_CORRECT = $line);  
	}
	if (lc($YES_CORRECT) eq "y") {

		print "WARNING: the default for the interface may not be what you want.\n";
		if ($systype eq "Linux") {
			$interface = "eth0";	# just an informed guess
		}
		print "WARNING: use #ifconfig -a to determine interfaces.\n";
		print "Please enter the input interface name to sniff from: [$interface] ";
		$line = <STDIN>;
		unless ($line =~ /^\n/) {
		  	chomp($interface = $line);  
		}
		if ($line eq "lo0") {
			print "WARNING: interface lo0 is only for testing\n";
		}
		if ($line eq "lo") {
			print "WARNING: interface lo is only for testing\n";
		}
		print "input interface is $interface\n";

		print "\nPlease enter directory for probe output files (mon.lite, etc.): [$monlite] ";
		$line = <STDIN>;
		unless ($line =~ /^\n/) {
		  	chomp($monlite = $line);  
		}
		print "probe output directory name is: $monlite\n";
 
		print "\nCreating bin/ourmon.sh driver for startup of ourmon.\n";
		# create ourmon.sh driver to run ourmon with previous parameters
		chdir("bin");
		write_ourmon_shell($interface, $monlite, $ourmon_conf);
		chdir("..");

		print "ourmon.sh placed in ourmon bin for ourmon front-end/probe startup\n";
		print "./ourmon.sh start\n"; 

		# install ourmon.sh in /etc/init.d or whatever for boottime install
		# XXX.  this is same script as in bin/ourmon.sh
		$YES_CORRECT="y";
		$path = $INSTALL_PREFIX . "/" . "bin/ourmon.sh";
		if ($systype eq "FreeBSD") {
			print "\ncopy the startup script (bin/ourmon.sh) to /usr/local/etc/rc.d for boot startup? [y] ";
			$line = <STDIN>;
			unless ($line =~ /^\n/) {
		  		chomp($YES_CORRECT = $line);  
			}
			if (lc($YES_CORRECT) eq "y") {
				system("cp $path /usr/local/etc/rc.d");
				system("chmod +x /usr/local/etc/rc.d/ourmon.sh");
			}
		}
		else {
			if ($systype eq "Linux") {
				print "WARNING: this is a gross guess and it may be best handled by you yourself!\n";
				print "WARNING: linux has at least two major variations in distributions in this area!\n";
				print "\ninstall the startup script (bin/ourmon.sh) in /etc somewhere for boot startup? [y] ";
				# check for slackware type of install
				# see if /etc/rc.d/*local exists then simply append to it
				# else assume debian/fedora type with /etc/init.d and /etc/rc* for states
				$line = <STDIN>;
				unless ($line =~ /^\n/) {
					chomp($YES_CORRECT = $line);  
				}
				if (lc($YES_CORRECT) eq "y") {
					# simple install
					if (-e "/etc/rc.d/rc.local") {
						system("echo $path >> /etc/rc.d/rc.local");
					}
					else {
						if (-e "/etc/init.d") {
							print "LINUX NOTE: for boot: /etc/init.d exists \n";
							print "\tand we place ourmon.sh there with rc linkages\n";
							system("cp $path /etc/init.d");
							system("chmod +x /etc/init.d/ourmon.sh");
							make_linux_rc_linkage("/etc/init.d/ourmon.sh");
						}
						else {
							print "Sorry - haven't a clue.  do something with bin/ourmon.sh\n";
						}
					}
				}
			}
			else {
				print "Sorry - haven't a clue.  do something with bin/ourmon.sh\n";
			}
		}
		print "ourmon front-end install complete\n";
	}
	if ($ourmon_ok) {
		print "ourmon front-end build worked\n";
		print "\nYou should now run $FINAL_SCRIPT_DIR/bin/ourmon.sh to start ourmon\n";
		print "\ne.g., # $FINAL_SCRIPT_DIR/bin/ourmon.sh start\n";
		print "\nYou can use ourmon.sh stop to stop ourmon\n";
	}
	print "\n";
}

print "part 2: install the back-end, omupdate.pl, etc. (web part)? [y] ";
$line = <STDIN>;
unless ($line =~ /^\n/) {
  	chomp($BACKEND = $line);  
}

# install of backend, omupdate.pl, drawtopn, logging
# 
if (lc($BACKEND) eq "y") {
	print "Back-end configuration phase started ######################################\n";

	# WEBDIR
	print "We need a local web directory for generated web output.\n";
	print "hint: the webpath given here is a guess: give the CORRECT base web directory with /ourmon at the end\n";
	print "enter absolute web server web path directory: [$webpath] ";
	$line = <STDIN>;
	unless ($line =~ /^\n/) {
	  	chomp($webpath = $line);  
	}
	print "your output web path is: $webpath\n";

	print "\nDo you want to create the web directory for ourmon?";
	print "\nHINT: good idea if it doesn't exist. [y] ";
	$YES_CORRECT = "y";
	$line = <STDIN>;
	unless ($line =~ /^\n/) {
	  	chomp($YES_CORRECT = $line);  
	}
	if (lc($YES_CORRECT) eq "y") {
		system("mkdir $webpath");
	}
	# drop in symlink for webpath in mrourmon path
	system("ln -s $webpath web.pages"); 

	# assume logging is on
	$LOGGING = "on";

	# cp back-end scripts and associated files to bindir and webdir
	# as appropriate
	
	my $path = $INSTALL_PREFIX . "/" . "bin";
	do_webcode($path,$webpath);
	
	# note: we now have bin/*.sh etc scripts installed
	print "\nINFO only: also setting up logging directory (if needed)\n";
	if (!-d "logs/mon.lite") {
#print "DEBUG: mklogdir.sh $INSTALL_PREFIX\n";
		system("bin/mklogdir.sh $INSTALL_PREFIX");
	}
	# note this sets up needed "daily" symlinks
	my $path = $INSTALL_PREFIX . "/" . "logs";
#print "DEBUG: daily.pl.sh $path\n";
	system("bin/daily.pl $path");
	print "hit CR to continue: ";
	$line = <STDIN>;

	# determine probe output directory
	print "\nIf different, enter front-end output file directory absolute path: [$monlite] ";
	$line = <STDIN>;
	unless ($line =~ /^\n/) {
	  	chomp($monlite = $line);  
	}
	print "probe output file path (back-end input/s) is $monlite\n\n";

	# COPY OVER HTML FILES TO WEBDIR
	print "\nNow we copy supplied .html files to the web directory for later editing\n";
	print "do you want to copy base web files to the web directory? [y] ";
	$YES_CORRECT = "y";
	$line = <STDIN>;
	unless ($line =~ /^\n/) {
	  	chomp($YES_CORRECT = $line);  
	}
	if (lc($YES_CORRECT) eq "y") {
		system("cp src/web.html/* $webpath");
		system("cp INSTALL $webpath/INSTALL.txt");
	}

	# tell where rrdbase is localed.
	print "\nINFO only: setting up local rrdbase directory at $rrdbase\n";
	print "\your runtime rrds get stored in this directory, along with the rrd error log file\n";
	print "\if you create new BPF filters, check rrdbase/ourmon.log for errors.\n";
	print "hit CR to continue: ";
	$line = <STDIN>;
	
	# get udp weight threshold
	print "\nWe need a UDP weight threshold for UDP scan alerts\n";
	print "what should the weight be (default is given): [$udpweight] ";
	$line = <STDIN>;
	unless ($line =~ /^\n/) {
	  	chomp($udpweight = $line);  
	}
	print "\n";

        #$sysdir = File::Basename::dirname($syslog_ip);
        #$sysbase = File::Basename::basename($syslog_ip);

	# we need to slightly modify various back-end scripts
	chdir ("bin");	
	fix_backend_scripts($ourpath, $webpath, $udpweight);
	chdir ("..");	

	# ask if crontab file should be appended to /etc/crontab
	# create crontab variables

	# TBD. see if /etc/crontab exists else see if
	# /var/spool/cron/crontab/root exists
	# else give up
	# note the latter is non-vixie and the format has no user
	$YES_CORRECT="y";	
	$vixie = 1;
	$cronlocation = "/etc/crontab";
	if (-e "/var/spool/cron/crontab/root") {
		$vixie = 0;
		$cronlocation = "/var/spool/cron/crontab/root";
	}
	print "\nInstall backend crontab commands in $cronlocation (default answer y)?: [$YES_CORRECT] ";
	$line = <STDIN>;
	unless ($line =~ /^\n/) {
	  	chomp($YES_CORRECT = $line);  
	}
print "$YES_CORRECT\n";
	if ($vixie) {
		create_crontab_file($LOGGING, $YES_CORRECT, $crontab_file, $cronlocation);
	}
	else {
		create_nonvixie_crontab_file($LOGGING, $YES_CORRECT, $crontab_file, $cronlocation);
	}
}

print "\nourmon system config complete\n";
print "see INSTALL for post-config sanity checking\n";

# end of script main
exit (0);

#
# create bin/ourmon.sh script to drive running ourmon
# at boot.
#
# input params:
#	interface:
#	mon.lite pathname:
#	config file
sub write_ourmon_shell
{
	my $interface = $_[0];
	my $outputfile = $_[1];
	my $config = $_[2];
	my $path = $ourpath . "/" . "bin/ourmon";
	my $shell = "/bin/sh";

	if ($systype eq "FreeBSD") {
		write_bsd_shell($interface, $outputfile, $config);
		return;
	}

	write_linux_shell($interface, $outputfile, $config);
	return;
}

# write bsd 5.x style startup script
# which can be used on 4.X systems as well. 
#
sub write_bsd_shell
{
	my $interface = $_[0];
	my $outputfile = $_[1];
	my $config = $_[2];
	my $path = $ourpath . "/" . "bin/ourmon";
	my $shell = "/bin/sh";

  	open(HANDLE, ">$ourmon_script") || die "Cannot open $ourmon_script";

	my $bsdsysctl1 = "debug.bpf_bufsize";
	my $bsdsysctl2 = "debug.bpf_maxbufsize";
	if ($fbsd6) {
		$bsdsysctl1 = "net.bpf.bufsize";
		$bsdsysctl2 = "net.bpf.maxbufsize";
	}

	print HANDLE "#!$shell\n";
 	print HANDLE<<END_OF_TEXT;
# ourmon.sh (auto-generated)
# the sysctl variables exist to reduce dropped packets
# note: on BSD, you may need a recent version of libpcap for this to work
# get it from: www.tcpdump.org
# On BSD: compare sysctl -a outputs to /var/log/messages, bpf bufsizes should match! 
start_om()
{
	sysctl -w $bsdsysctl1=8388608
	sysctl -w $bsdsysctl2=8388608 
END_OF_TEXT
	print HANDLE "\t$path -a 30 -s 256 -f $config -i $interface -D $outputfile &\n";

 	print HANDLE<<END_OF_TEXT;
	echo -n ' ourmon'
}

stop_om()
{
        kill -9 `cat /var/run/ourmon.pid`
        echo -n ' ourmon'
}

restart_om()
{
        stop_om
        start_om
}

_action="\$1";

if [ -z "\$_action" ]; then
        _action="start"
fi

case \$_action in
start)
        start_om
        ;;
stop)
        stop_om
        ;;
restart)
        restart_om
        ;;
*)      echo "Usage: ourmon.sh {start|stop|restart}" >&2
        exit 64
esac
exit 0
END_OF_TEXT

	if (system "chmod +x $ourmon_script") {
	  	print "Unable to chmod $ourmon_script\n";
	}
}

# write linux startup script
# not very different from bsd startup script
# but convenient to have as separate function for coding.
#
sub write_linux_shell
{
	my $interface = $_[0];
	my $outputfile = $_[1];
	my $config = $_[2];
	my $path = $ourpath . "/" . "bin/ourmon";
	my $shell = "/bin/sh";

  	open(HANDLE, ">$ourmon_script") || die "Cannot open $ourmon_script";

	print HANDLE "#!$shell\n";
 	print HANDLE<<END_OF_TEXT;
# ourmon.sh (auto-generated)
# the sysctl variables exist to reduce dropped packets
# note: on BSD, you may need a recent version of libpcap for this to work
# get it from: www.tcpdump.org
# On BSD: compare sysctl -a outputs to /var/log/messages, bpf bufsizes should match! 
start_om()
{
	sysctl -w net.core.rmem_default=8388608
	sysctl -w net.core.rmem_max=8388608
END_OF_TEXT
	print HANDLE "\t$path -a 30 -s 256 -f $config -i $interface -D $outputfile &\n";

 	print HANDLE<<END_OF_TEXT;
	echo -n ' ourmon'
}

stop_om()
{
        kill -9 `cat /var/run/ourmon.pid`
        echo -n ' ourmon'
}

restart_om()
{
        stop_om
        start_om
}

_action="\$1";

if [ -z "\$_action" ]; then
        _action="start"
fi

case \$_action in
start)
        start_om
        ;;
stop)
        stop_om
        ;;
restart)
        restart_om
        ;;
*)      echo "Usage: ourmon.sh {start|stop|restart}" >&2
        exit 64
esac
exit 0
END_OF_TEXT

	if (system "chmod +x $ourmon_script") {
	  	print "Unable to chmod $ourmon_script\n";
	}
}

# install backend scripts and bar web files
# note: this installs bar files in web dir too.
#

sub do_webcode
{
	my $ourbin = $_[0];		# bin path
	my $ourweb = $_[1];		# web path

	my $bpath="BINDIR=$ourbin";
	my $wpath="WEBDIR=$ourweb/bard";  # for installing bar files

	mkdir ("$ourweb/bard");		  # make sure it exists as web subdir
	my $rc = 0;
	chdir("src/web.code");
	$rc = system("make -f Makefile install \"$bpath\" \"$wpath\"");
#print $rc, "\n";
	chdir("../..");

}

#
# /tmp/crontab.ourmon.xxx 
#
# create crontab entries for graphics-engine/back-end
# 
# cron is how the back-end works.
#
# batchip.sh, batchipall.sh, omupdate.sh, daily.pl
#
sub create_crontab_file() 
{
	my $logging = $_[0];    # add logging parts
	my $install = $_[1];	# install in /etc/crontab 
	my $file    = $_[2];    # name of tmp file 
	my $cronlocation = $_[3];  # where root crontab lives
	my $vixie = 1;

	# just assume logging is on
	#
	my $binpath = $ourpath . "/" . "bin/";
	my $omupdatepath = $binpath . $omupdate_script;
	my $dstring = $binpath . "daily.pl " . $logpath;
  	open(HANDLE, ">$file") || die "Cannot open $file";

	print HANDLE "############## ourmon crontab entries ###############\n";
	$batchip_file = $binpath . $batchip_file;
	$batchipall_file = $binpath . $batchipall_file;
	my $basic = "*/1	*	*	*	*	root	$omupdatepath\n";
	print HANDLE "# run ourmon back-end omupdate.pl etc. per minute\n";
 	print HANDLE "$basic";
	print HANDLE "# batchip.sh - hourly log summary\n";
	print HANDLE "0	*	*	*	*	root	$batchip_file\n";
	print HANDLE "# batchipall.sh - roll over daily summary logs at almost midnight\n";
	print HANDLE "50	23	*	*	*	root	$batchipall_file\n";
	print HANDLE "# daily.pl - re init next day logs at midnight\n";
	print HANDLE "0	0	*	*	*	root	$dstring\n";
	close HANDLE;
	# concat to /etc/crontab if desired
	if ($install eq "y") {
		system("cat $file >> $cronlocation");
		system("rm -f $file");
	}
	else {
		print "NOTE: suggested root crontab additions in $file\n";
		print "NOTE: you may concatenate that file to $cronlocation\n";
	}
}

#
# /tmp/crontab.ourmon.xxx 
#
# create crontab entries for graphics-engine/back-end
# 
# cron is how the back-end works.
#
# batchip.sh, batchipall.sh, omupdate.sh, daily.pl
#
sub create_nonvixie_crontab_file() 
{
	my $logging = $_[0];    # add logging parts
	my $install = $_[1];	# install in blah blah blah
	my $file    = $_[2];    # name of tmp file 
	my $cron    = $_[3];    # name of crontab location in real world
	my $vixie = 1;


############## ourmon crontab entries ###############
# daily.pl - re init next day logs at midnight
#0 0 * *  * /home/mrourmon/bin/daily.pl /home/mrourmon/logs 1>/dev/null
# run ourmon back-end omupdate.pl etc. per minute
#*/1 *  *  *  * /home/mrourmon/bin/omupdate.sh 1>/dev/null
# batchip.sh - hourly log summary
#0  *  *  *  * /home/mrourmon/bin/batchip.sh 1>/dev/null
# batchipall.sh - roll over daily summary logs at almost midnight
#59 45  *  *  *  /home/mrourmon/bin/batchipall.sh 1>/dev/null


	my $binpath = $ourpath . "/" . "bin/";
	my $omupdatepath = $binpath . $omupdate_script;
	my $dstring = $binpath . "daily.pl " . $logpath;
  	open(HANDLE, ">$file") || die "Cannot open $file";
	print HANDLE "############## ourmon crontab entries ###############\n";
	$batchip_file = $binpath . $batchip_file;
	$batchipall_file = $binpath . $batchipall_file;
	my $basic = "*/1 * * * * $omupdatepath 1>/dev/null\n";
	print HANDLE "# run ourmon back-end omupdate.pl etc. per minute\n";
 	print HANDLE "$basic";
	print HANDLE "# batchip.sh - hourly log summary\n";
	print HANDLE "0	* * * * $batchip_file 1>/dev/null\n";
	print HANDLE "# batchipall.sh - roll over daily summary logs at almost midnight\n";
	print HANDLE "50 23 * * * $batchipall_file 1>/dev/null\n";
	print HANDLE "# daily.pl - re init next day logs at midnight\n";
	print HANDLE "0	0 * * * $dstring 1>/dev/null\n";
	close HANDLE;
	# concat to /etc/crontab if desired
	if ($install eq "y") {
		system("cat $file >> $cron");
	}
	else {
		print "NOTE: suggested root crontab additions put in $file\n";
		print "NOTE: you may concatenate that file to $cron\n";
	}
}


# sigh.
# try to make ourmon happen last
#
sub make_linux_rc_linkage()
{
	my $bootfile = $_[0]; 

	system("ln -s $bootfile /etc/rc0.d/K120ourmon");
	system("ln -s $bootfile /etc/rc1.d/K120ourmon");
	system("ln -s $bootfile /etc/rc2.d/S120ourmon");
	system("ln -s $bootfile /etc/rc3.d/S120ourmon");
	system("ln -s $bootfile /etc/rc4.d/S120ourmon");
	system("ln -s $bootfile /etc/rc5.d/S120ourmon");
	system("ln -s $bootfile /etc/rc6.d/K120ourmon");
}

#
# use sed to edit various back-end scripts on the fly
# and for the most part give them correct paths to the ourmon bin.
# thus these scripts allow ourmon to be installed anywhere.
#
#	1. batchipall.sh  WEBDIR=/home/mrourmon/web.pages
#	2. batchip.sh     BASEDIR=/home/mrourmon
#	3. omupdate.sh  
#		BASEDIR=/home/mrourmon
#		UDPWEIGHT=something
#
# XXX. don't make changes if they are not needed.
#
sub fix_backend_scripts
{
	my $ourpath = $_[0];
	my $webpath = $_[1];
	my $udpweight = $_[2];

 	my $oldline="WEBDIR=.*\$";
 	sed_line_munge("batchipall.sh", $oldline, "WEBDIR=$webpath");

 	my $oldline="BASEDIR=.*\$";
 	sed_line_munge("batchip.sh", $oldline, "BASEDIR=$ourpath");

	my $oldline="BASEDIR=.*\$";
	sed_line_munge("omupdate.sh", $oldline, "BASEDIR=$ourpath");

	my $oldline="UDPWEIGHT=.*\$";
	sed_line_munge("omupdate.sh", $oldline, "UDPWEIGHT=$udpweight");

	system("chmod +x *.sh *.pl");
}

#$OLDPATH="BASEDIR=/home/mrourmon";
#$NEWPATH="BASEDIR=/usr/local/tmp";
# sedmunge("foo.sh", $OLDPATH, $NEWPATH);
# replace one line in a file with a substitution pattern
# note we must make sed happy so if we are using a directory
# path of the nature /a/b we turn it into \/a\/b
# 
# TBD use mktemp of some sort.
sub sed_line_munge
{
	my $file=$_[0];
	my $oldpattern=$_[1];
     	$oldpattern =~ s/\//\\\//g;
	my $newpattern=$_[2];
     	$newpattern =~ s/\//\\\//g;
	# pattern match on the patterns to turn slash into backslash slash
	my $sbuf=sprintf("sed \"s/^$oldpattern/$newpattern/\" $file > /tmp/our_tmp.xxx");
	#print "$sbuf\n";
	system($sbuf); 
	system("mv /tmp/our_tmp.xxx $file");
}

sub guess_bsd_if
{
	my $f1;
	open(BSDIF, "ifconfig -l |");
	while(<BSDIF>) {
		chop;
        	($f1) = split;
	}
	close(BSDIF);
	return($f1);
}
