/*
 * cprogram.c
 *
 * rough C equivalents of common BPF expressions in ourmon
 *
 */

/*
 * Copyright (c) 2004 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef	HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <pcap.h>

#include  "ourmon.h"
#include  "filter.h"
#include  "cprogram.h"

/* 
 * c_network()
 *
 * roughly: net ip/mask
 *
 * take the src and dst in turn, and see if they belong
 * in the range.  reverse and check, else fail.
 *
 * config assumption:  the config'ed net & mask have already been anded.
 *	therefore we need only and target net & mask here and compare to
 *	the config net value.
 *
 * returns:
 *	1 - match
 *	0 - no match
 */
int 
c_network(struct t_entry *te, struct bpf_cparams *b) 
{
	struct in_addr src, dst;
	extern int debug;

#ifdef MDEBUG
	if (debug)
		printf("c_network %x %x %x\n", te, b, b->net.s_addr);
#endif
	src.s_addr = te->src.s_addr & b->netmask.s_addr;
	if ( src.s_addr == b->net.s_addr ) {
		return(1);
	}
	dst.s_addr = te->dst.s_addr & b->netmask.s_addr;
	if ( dst.s_addr == b->net.s_addr ) {
		return(1);
	}
	return(0);
}

/*
 * determine if packet is ping
 */
int 
c_ping(struct t_entry *te, struct bpf_cparams *b) 
{
	extern int debug;
	
#ifdef MDEBUG
	if (debug) {
		printf("c_ping called\n");
		dump_te(te);
	}
#endif

        if (te->proto == IPPROTO_ICMP) {
		if ((te->sport == 8) || (te->sport == 0)) {
			return(1);
		}
	}
	return(0);
}

/*
 * determine if packet matches major/minor ICMP code
 */
int 
c_icmp(struct t_entry *te, struct bpf_cparams *b) 
{
	extern int debug;

#ifdef MDEBUG
	if (debug) {
		printf("c_icmp called\n");
		dump_te(te);
	}
#endif
        if (te->proto == IPPROTO_ICMP) {
		if ((te->sport == b->icmp_major) && (te->dport == b->icmp_minor)) {
			return(1);
		}
	}
	return(0);
}

/*
 * determine if packet tcp syn or fin or reset flags
 */
int 
c_tcpflag(struct t_entry *te, struct bpf_cparams *b) 
{
	extern int debug;

#ifdef MDEBUG
	if (debug) {
		printf("c_tcpflag called\n");
		dump_te(te);
	}
#endif
        if (te->proto == IPPROTO_TCP) {
		if (b->synflag && te->synflag) 
			return(1);
		else if (b->finflag && te->finflag) 
			return(1);
		else if (b->resetflag && te->resetflag) 
			return(1);
	}
	return(0);
}

/*
 * determine if packet is with a tcp port range which is inclusive
 */
int
c_tcprange(struct t_entry *te, struct bpf_cparams *b)
{
        extern int debug;

#ifdef MDEBUG
        if (debug) {
                printf("c_tcprange called\n");
                dump_te(te);
        }
#endif
        unsigned int plo = (unsigned int) b->tports[0];
        unsigned int phi = (unsigned int) b->tports[1];
        if (te->proto == IPPROTO_TCP) {
                if ((te->sport >= plo) && (te->sport <= phi))
                        return(1);
                if ((te->dport >= plo) && (te->dport <= phi))
                        return(1);
        }
        return(0);
}

/*
 * determine if packet is with a tcp port range which is inclusive
 */
int
c_udprange(struct t_entry *te, struct bpf_cparams *b)
{
        extern int debug;

#ifdef MDEBUG
        if (debug) {
                printf("c_udprange called\n");
                dump_te(te);
        }
#endif
        unsigned int plo = (unsigned int) b->uports[0];
        unsigned int phi = (unsigned int) b->uports[1];
        if (te->proto == IPPROTO_UDP) {
                if ((te->sport >= plo) && (te->sport <= phi))
                        return(1);
                if ((te->dport >= plo) && (te->dport <= phi))
                        return(1);
        }
        return(0);
}

/*
 * determine if packet matches a given udp or tcp port
 * note: this is the logical equivalent of an or of all of the ports
 */
int
c_ports(struct t_entry *te, struct bpf_cparams *b)
{
        extern int debug;
	int i;

        if (debug) {
                printf("PORTS: c_ports called\n");
                dump_te(te);
        }
        if (te->proto == IPPROTO_TCP) {
		for ( i = 0; i < b->tportcount; i++) {
			if (te->sport == b->tports[i]) {
/*
printf("TCP sport i=%d, MATCH %d %d\n", i, te->sport, b->tports[i]);
*/
				return(1);
			}
			if (te->dport == b->tports[i]) {
/*
printf("TCP dport i=%d, MATCH %d %d\n", i, te->dport, b->tports[i]);
*/
				return(1);
			}
		}
        }
        else if (te->proto == IPPROTO_UDP) {
		for ( i = 0; i < b->uportcount; i++) {
			if (te->sport == b->uports[i]) {
/*
printf("UDP sport i=%d, MATCH %d %d\n", i, te->sport, b->uports[i]);
*/
				return(1);
			}
			if (te->dport == b->uports[i]) {
/*
printf("UDP dport i=%d, MATCH %d %d\n", i, te->dport, b->uports[i]);
*/
				return(1);
			}
		}
        }
        return(0);
}

/*
 * determine if packet matches a particular ip protocol
 */
int
c_ipproto(struct t_entry *te, struct bpf_cparams *b)
{
        extern int debug;
	int i;

#ifdef MDEBUG
        if (debug) {
                printf("c_icmp called\n");
                dump_te(te);
        }
#endif
	/* due to paranoia about ip proto 0
	 * being used, we make "ip" mean any packet
	 * 0 is a legal value, but only gets protos
	 * with a 0 in the proto field.
	 */
	if (b->ipproto < 0) {
		return(1);
	}
        if (te->proto == b->ipproto) {
		return(1);
        }
        return(0);
}
