/*
 *
 * noipanalyze.c
 *
 * runs traditional fixed hardwired filters that are NOT
 * ip specific.
 *
 */

/*
 * Copyright (c) 2005 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef	HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <pcap.h>
#ifdef	TIME_WITH_SYS_TIME
#include <time.h>
#endif

#include "ourmon.h"
#include  "filter.h"

u_char     etherbroadcastaddr[6] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };

/*
 * analyze_nonip
 *
 * deals with low-level hw filters when eh exists.
 *	fixed_cast
 *	fixed_size
 *
 * that are not IP specific.  However it should only be called
 * when an ethernet header exists.  analyze_ip will deal with those
 * filters (localhost case) if there is no L2 sane header.
 *
 * returns:
 *	NULL at this time.
 */

void analyze_nonip( char *p, int, int, struct ether_header *, struct t_entry *);

void
analyze_nonip(p, length, caplength, eh, te)
	char *p;		/* L4 portion */
	int length;		/* length - of L4 portion minus link */
	int caplength;		/* actual length of packet including link */
	struct ether_header *eh;	/* ethernet header */
	struct t_entry *te;	
{
	int etype;
	int hlen;
	register u_char *cp;

	if (!eh)
		return;

	etype = ntohs(eh->ether_type);

	/* process ethernet info, just in case.  we do this to make the
	 * fixed_cast filter happy.
	*/
	te->eflag = 0;			
	if (eh) {
		te->eflag = 1;			/* this means we have an eh */
		te->bflag = te->mflag = 0;	/* assume unicast */
		/* check if broadcast, else multicast, default
		 * to unicast
		 */
		if ( bcmp(&eh->ether_dhost, etherbroadcastaddr, 6) == 0) {
			te->bflag = 1;
		}
#ifdef SOLARIS
                else if ((eh->ether_dhost.ether_addr_octet[0] & 1) == 1) {
#else
		else if ((eh->ether_dhost[0] & 1) == 1) {
#endif
			te->mflag = 1;
		}
		/* this function uses ethernet stats only via the te structure
		 */
		if (fixed_cast.configured)
	 		fixed_cast_filter_l2(caplength, te);
	}

	/* process for all packet types including errors
	*/
	if (fixed_size.configured && eh) {
		if (caplength <= 77) {
			fixed_size.tiny++;
			te->sizeindex = 0;
		}
		else if (caplength <= 110) {
			fixed_size.tiny++;
			te->sizeindex = 1;
		}
		else if (caplength <= 250) {
			fixed_size.small++;
			te->sizeindex = 2;
		}
		else if (caplength <= 500) {
			fixed_size.small++;
			te->sizeindex = 3;
		}
		else if (caplength <= 1000) {
			fixed_size.medium++;
			te->sizeindex = 4;
		}
		else {
			fixed_size.large++;
			te->sizeindex = 5;
		}
	}

	etype = ntohs(eh->ether_type);

	if (etype == 0x800) {
		fixed_l2proto.ip++;
	}
	else if (etype == 0x806) {
		fixed_l2proto.arp++;
	}
	else if (etype == 0x86dd) {
		fixed_l2proto.ipv6++;
	}
	else {
		fixed_l2proto.xtra_bytes++;
	}
	return;
}
