#ifndef PATMATCH_H
#define PATMATCH_H
#include <pcre.h>

#define MAX_TAG_LEN 9
#define MAX_PAT_LEN 9999
#define MAX_U_SHORT 65535
#define MAX_PACKETS 8
#define MAX_BYTES 2048

#define FALSE 0
#define TRUE 1

#define NO 0
#define YES 1

#define aVALUE 0x1
#define bVALUE 0x2
#define cVALUE 0x4
#define dVALUE 0x8
#define eVALUE 0x10
#define fVALUE 0x20
#define gVALUE 0x40
#define hVALUE 0x80
#define iVALUE 0x100
#define jVALUE 0x200
#define kVALUE 0x400
#define lVALUE 0x800
#define mVALUE 0x1000
#define nVALUE 0x2000
#define oVALUE 0x4000
#define pVALUE 0x8000
#define qVALUE 0x10000
#define rVALUE 0x20000
#define sVALUE 0x40000
#define tVALUE 0x80000
#define uVALUE 0x100000
#define vVALUE 0x200000
#define wVALUE 0x400000
#define xVALUE 0x800000
#define yVALUE 0x1000000
#define zVALUE 0x2000000

#define AVALUE 0x1
#define BVALUE 0x2
#define CVALUE 0x4
#define DVALUE 0x8
#define EVALUE 0x10
#define FVALUE 0x20
#define GVALUE 0x40
#define HVALUE 0x80
#define IVALUE 0x100
#define JVALUE 0x200
#define KVALUE 0x400
#define LVALUE 0x800
#define MVALUE 0x1000
#define NVALUE 0x2000
#define OVALUE 0x4000
#define PVALUE 0x8000
#define QVALUE 0x10000
#define RVALUE 0x20000
#define SVALUE 0x40000
#define TVALUE 0x80000
#define UVALUE 0x100000
#define VVALUE 0x200000
#define WVALUE 0x400000
#define XVALUE 0x800000
#define YVALUE 0x1000000
#define ZVALUE 0x2000000


/*structure to be used for regular expression tag storage -
 * a tag is a one character ([a-zA-Z]) value associated with a
 * regular expression in the configuration file.
 * The storage in this structure is as follows:
 * upper(32bit):------ZYXWVUTSRQPONMLKJIHGFEDCBA
 * lower(32bit):------zyxwvutsrqponmlkjihgfedcba
 * */
struct tags {
	unsigned int upper;
	unsigned int lower;
};


/*The main pattern storage. The tag and pattern values are provided by the
 * program using this library. The rest is computed from these values. */
struct patdescr {
	char tag; //the character tag - maybe not needed?
	struct tags apptags; //the bit representation of the character tag
	u_short protocol;
	u_int maxpkts;
	u_int maxbytes;
	u_short removeNull;
	char pattern[MAX_PAT_LEN];
	pcre *compiled;
	pcre_extra *extrainfo;
	struct patdescr *next;
};


/*public functions*/
int patmatch_addPattern(char *proto, char tag, char *maxpkts, char *maxbytes, u_short removenull, char *pattern,struct patdescr **head);
int patmatch_matchAll(u_short protocol,struct tags *apps, char *payload, struct patdescr *head, int payloadlen);
int patmatch_matchWithHistory(u_short protocol, struct tags *apps, unsigned char *payload, struct patdescr *head, int payloadlen, struct tags *attempted, struct tags *appres, unsigned int pkts, unsigned int bytes);
/*struct tags patmatch_matchGiven(struct tags apps, char *payload); //TODO*/
void patmatch_printPatterns(struct patdescr *head);
void patmatch_printTags(struct tags apptags);
/*below is called by ourmon before it initiates a match - will only match*/
/*if there are patterns to match to*/
int patmatch_isEmpty(struct patdescr *head);

/*private functions*/
u_short patmatch_getProto(char *proto);
char *patmatch_getProtoName(u_short proto);
struct tags patmatch_tagToBits(char tag);
int patmatch_patternChecked(char **pattern);
int patmatch_tagChecked(char tag);
/*
int patmatch_matchWrapper(struct patdescr *head, unsigned char *payload, int payloadlen);
*/

#endif
