/*
 * Copyright (c) 2007 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <signal.h>
#include <stdio.h>

// signals are blocked if they are members of the current signal mask set.

#ifdef NOTUSED
// block 'sig'
// SIG_BLOCK: the new mask is the union of current mask and the specified set.
/*
void inline signal_block(int sig)
{
	sigset_t newmask;
	sigemptyset(&newmask);
	sigaddset(&newmask, sig);
	sigprocmask(SIG_BLOCK, &newmask, NULL);
}

// unblock 'sig'
// SIG_UNBLOCK: new mask is the intersection of current mask and the
// complement of the specified set.
void inline signal_unblock(int sig)
{
	sigset_t newmask;
	sigemptyset(&newmask);
	sigaddset(&newmask, sig);
	sigprocmask(SIG_UNBLOCK, &newmask, NULL);
}
*/
#endif

static sigset_t newmask, oldmask;

void signal_block();
void signal_unblock();

void signal_block()
{
	sigemptyset(&newmask);
	sigaddset(&newmask, SIGALRM);
	sigaddset(&newmask, SIGTERM);

	//sigaddset(&newmask, SIGUSR1);
	sigprocmask(SIG_BLOCK, &newmask, &oldmask);
}

void signal_unblock()
{
	sigprocmask(SIG_SETMASK, &oldmask, NULL);
}

#ifdef NOTUSED

// sets the signal handler for 'sig'
inline void signal_set(int sig, void (*func)(int sig))
{
	struct sigaction act;
	act.sa_handler = func;
	sigemptyset (&act.sa_mask);
	act.sa_flags = 0;
	sigaction(sig, &act, NULL);
}

#endif
