/*
 * Copyright (c) 2007 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef THREAD
// thread.c
// primitives for thread manipulation in ourmon
//   - linux: use clone to create threads with their own stack.
//   - bsd: use rfork to do the same

#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>

#include "spinlock.h"
#include "sync.h"
#include "signal.h"
#include "thread.h"

#ifdef LINUX
#include <sched.h>
#endif

extern void signal_block();
extern void signal_unblock();

// Creates a thread, returns pid and pointer to stack
int thread_create(int (*func)(void *), thread_t * me)
{
	void * stack;
	pid_t pid;

	// block signal so that parent will not be interupted:
	signal_block();
	spinlock(&locks[LOCK_MALLOC]);
	if (!(stack = malloc(STACKSIZE))) {
		perror("Malloc: could not allocate stack");
		exit(1);
	}
	unlock(&locks[LOCK_MALLOC]);
	signal_unblock();

#ifdef LINUX
	// Call the clone system call to create the child thread
	pid = clone(func, stack + STACKSIZE,
		CLONE_FS | CLONE_FILES | CLONE_SIGHAND | CLONE_VM, me);
	if (pid == -1) {
		perror("clone() error");
		exit(2);
	}
#else

	// Call the rfork_thread  system call to create the child thread
	pid = rfork_thread(RFPROC|RFMEM, stack + STACKSIZE,func,me);
	if (pid == -1) {
		perror("rfork() error");
		exit(2);
	}

/* MAX OS X
      int rc = pthread_create(&pid,  NULL, func, me);
      if (rc) {
         perror("pthread_create() error\n");
         exit(2);
      }
*/
#endif
	me->pid = pid;
	me->stack = stack;
	return pid;
}

void thread_destory(thread_t *t)
{
	if (t) {
		free(t->stack);
	}
}

#endif
