/**	drawbar.c
 * 	Usage: drawbar -w <width> -t <tall> [-c <color>] -f <output>
 *	Description: draw a single horizontal bar that is T pixels tall by W pixels wide
 *	Author(s): Shane Michael Matthews
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <errno.h>
#include <sys/types.h>   
#include <sys/socket.h>   
#include <signal.h>
#include <setjmp.h>
#include <unistd.h>
#include "gd.h"
#include "gdfontl.h"
#include "gdfontmb.h"
#include "gdfonts.h"
#include "gdfontt.h"

#define BLACK		0
#define RED		1
#define YELLOW	2
#define ORANGE	3
#define BLUE		4

int main(int argc,char **argv ) 
{
	extern char *optarg;
	extern int optind, opterr;
	int background, op, tall = 0, width = 0, color = BLACK;
	FILE *out = NULL;
	gdImagePtr im;
	char * filename =  NULL;
	
	/* parse command line args */
	while ((op = getopt(argc, argv, "w:t:f:c:")) != EOF) {
		switch (op) {
			case 't': 
				tall = atoi(optarg);                         
				break;
			case 'w':
				width = atoi(optarg);
				break;
			case 'f':
				filename = optarg;
				break;
			case 'c':
				if (strcmp("blue",optarg) == 0 ) {
					color = BLUE;
				} else if (strcmp("red",optarg) == 0 ) {
					color = RED;
				} else if (strcmp("yellow",optarg) == 0 ) {
					color = YELLOW;
				} else if (strcmp("orange",optarg) == 0 ) {
					color = ORANGE;
				} else {
					color = BLACK;
				}
				break;
			case 'h':
			default:
				printf("Usage: drawbar -w <width> -t <tall> [-c <color>] -f <output>\n");
				exit(0);
		}
	}
	if ( width <= 0 || tall <= 0 || filename == NULL ) {
		printf("Usage: drawbar -w W -t T -f output.png\n");
		exit(0);
	}

	im = gdImageCreate(width,tall);
	
	if ( color == BLACK ) {
		background = gdImageColorAllocate(im, 0, 0, 0);
	} else if ( color == RED ) {
		background = gdImageColorAllocate (im,0xFF,0x00,0x00);
	} else if ( color == YELLOW ) {
		background = gdImageColorAllocate (im,0xFF,0xFF,0x00);
	} else if ( color == BLUE ) {
		background = gdImageColorAllocate (im,0x00,0x00,0xFF);
	} else if ( color ==ORANGE) {
		background = gdImageColorAllocate(im, 0xFF, 0x7F, 0x00);
	}
	
	//draw_frame(im, width, tall);
	out = fopen(filename, "wb"); //png_file
	gdImagePng(im, out);
	fclose(out);
	gdImageDestroy(im);
	exit(0);

}

/* some notes on colors:
	black  = gdImageColorAllocate (im,0x00,0x00,0x00);
	blue   = gdImageColorAllocate (im,0x00,0x00,0xFF);
	cyan   = gdImageColorAllocate (im,0x00,0xFF,0xFF);
	green  = gdImageColorAllocate (im,0x00,0xFF,0x00);
	yellow = gdImageColorAllocate (im,0xFF,0xFF,0x00);
	red    = gdImageColorAllocate (im,0xFF,0x00,0x00);
	brown  = gdImageColorAllocate (im,0x78,0x4B,0x00);
	white  = gdImageColorAllocate (im,0xFF,0xFF,0xFF);
	navy   = gdImageColorAllocate (im,0x00,0x00,0x7F);
	gray   = gdImageColorAllocate (im,0x7F,0x7F,0x7F);
	orange = gdImageColorAllocate(img, 0xFF, 0x7F, 0x00)
*/


