#! /usr/bin/perl

#
# ombatchip.pl
#
# input params:
#
# ombatchip.pl report_type fullpath_logfile relative_asciifile
#
# report_type, either icmp, pkts (topn_ip_pkts), or anything else
#
# purpose: sort the log file to find the largest flows
#	in the log file, and write a report to a 
#	text file.  
#
# input: fullpath_logfile - unix log file output produced
#	by omupdate.pl.  This is the top IP flows log,
#	named topn.log (unless the name was changed).
#
# output: relative_asciifile - report
#
# flow database scheme 
# ipflow{ flow id } = amount
# ipflow_time { flow id} = time
#
# thus we map flow_id (string) to (time, amount) pairs
#
# does a cd to web directory.
#
# TUPLES:
#	FLOWTUPLE
#

$debug = 0;

# globals for ourmon

################################
# things that may be tweaked
# max number of outputs to store in a report
my $asciitopn = 100;

################################
use Socket;

# set mon.lite input file - this is where it is found
use Time::Local;
# comment in for ctime, note that it overrides localtime
use Time::localtime;

# month list - look up internal value, based on external string tag 
#
%month = ('Jan', 0, 'Feb', 1, 'Mar', 2, 'Apr', 3, 'May', 4, 'Jun',
             5, 'Jul', 6, 'Aug', 7, 'Sep', 8, 'Oct', 9, 'Nov', 10, 'Dec', 11);

my $reporttype = $ARGV[0];

$icmpreport = 0;
if ($reporttype eq "icmp") {
	$icmpreport = 1;
}

$pktsreport = 0;
if ($reporttype eq "pkts") {
	$pktsreport = 1;
}


# ip topn log file
my $iplogfile = $ARGV[1];

# absolute output filename
my $ipascii = $ARGV[2];

################################################### 
$start_time=`date`;
$start_time=~ s/\n//g;

#  set rrd file names, error log file name, top N png file
#  name.  rrds and error log must have rrdbase as path
######################################################

$unixtime=time;

# parse the file and put info in 
# associative arrays
if ($debug) {
	print "parsing stage\n";
}
ipflow_parse();

#dumpFlows();

# sort it
if ($debug) {
	print "sorting stage\n";
}
ipflow_sortit();

# print it
if ($debug) {
	print "printing stage\n";
}
ipflow_printit();

exit(0);

# end of main

######################################################################
# functions

# globals
#	$iplogfile - input log file
#	%ipflow
#	%ipflow_time
sub ipflow_parse() { 
	
	my $curline;
	my $time;
	my $timeval;
	my $words;
	my $count;
	my $countpairs;
	my $fid;
	my $realtuples;
	my $tuplecount = 5;      # 5 fields per current tuple

	open(FILE, $iplogfile) || die "can't open $iplogfile: $!\n";
	while(<FILE>) {

		s/  / /g;
		$curline = $_;
		@time = split(/ /, $curline);
		#print "$time[0]\n";
		#print "$time[1]\n";
		#print "$time[2]\n";
		$curline =~ s/ //g;    
		$curline =~ s/\n//g;
		# convert greater than to zilch, until we figure out
		# how to escape it 
		$curline =~ s/\>//g;

		@words = split(/:/, $curline);

		# save the time stamp, and shift the array
		# past it and the tag
		$timeval = getTime($time[0], $time[1], $time[2]);
		$x = shift(@words);	# time 1 (ignored)
		$x = shift(@words);	# time 2	
		$x = shift(@words);	# time 3
		$x = shift(@words);	# topn_ip or related tag 
	
		$realtuples = shift(@words);	# total count of flows

		$count = $#words;	# number of remaining words post shift

		# number of pairs - xtra which we ignore
		$countpairs = $count / 5;  
		$countpairs--;			# we number from 0..max-1

		if ($realtuples == 0) {
			next;
		}

		for ($i = 0; $i < $countpairs; ) {
			$fid = $words[$i];
			$fcount = $words[$i+1];
			$uaphw = $words[$i+2];
			$uappl = $words[$i+3];
			$uapph = $words[$i+4];
			
			# if there is a hit in the ass. cache
			# for the flow id
			if ($ipflow{$fid}) {
	#printf("HIT %s, keys %d\n", $fid, keys(%ipflow));
	#			if ($fcount > $ipflow{$fid}) {
	#printf("replace %s\ old new %d %dn", $fid, $ipflow{$fid}, $fcount);
	#				$ipflow{$fid} = $fcount;
	#printf("keys %d\n", keys(%ipflow));
	#				$ipflow_time{$fid} = $timeval;
	#			}
				# add the flows together (total)
				# we want to know who the big dogs are
				# on the other hand, leave time alone
				# this means we get the start time
		
				# note ok if fcount is pkts, not bytes
				$ipflow{$fid} += $fcount;
				$ipflow_lasttime{$fid} = $timeval;
				$ipflow_instances{$fid} += 1;
				$ipflow_uaphw{$fid} |= $uaphw;
				$ipflow_uappl{$fid} |= $uappl;
				$ipflow_uapph{$fid} |= $uapph;
			}
			# else simply store it
			else {
				$ipflow{$words[$i]} = $words[$i+1]; 
				# so this is the start time
				$ipflow_time{$words[$i]} = $timeval;
				$ipflow_lasttime{$words[$i]} = $timeval;
				$ipflow_instances{$words[$i]} = 1;
				$ipflow_uaphw{$fid} = $uaphw;
				$ipflow_uappl{$fid} = $uappl;
				$ipflow_uapph{$fid} = $uapph;
			}
	#			printf("%d: %s, %s\n", $i, $words[$i],
	#				$words[$i+1]);
			$i = $i + 5;
		}

	}  # end loop
	close(FILE);
}

sub dumpFlows 
{
	foreach $key (keys %ipflow) {
		printf("key %s: flow %s, time %d\n",          
			$key, $ipflow{$key}, $ipflow_time{$key});
	}
}

sub ipflow_sortit()
{
	@sortedipflows = sort ipflow_by_value keys(%ipflow);
}

sub ipflow_by_value {
        if ($ipflow{$a} < $ipflow{$b}) {
                1;
        }
        elsif ($ipflow{$a} == $ipflow{$b}) {
                0;
        }
        elsif ($ipflow{$a} > $ipflow{$b}) {
                -1;
        }
}

sub ipflow_printit {
	my $flowid;
	my $tstring;
	my $tstring2;
	my $hits;
	my $bits;
	my $lhs_ip, $lhs_port, $rhs_ip, $rhs_port, $proto;
	my $dnsname;
	my $lflow;
	my $af;

	# plaintext file store
	open(OUTPUT_HANDLE, ">$ipascii") || die "can not open: $!\n";
	if ($icmpreport) {
		print OUTPUT_HANDLE "start log time          : instances: flow (ip_src->ip_dst[major/minor code]): avg flow bits/sec\n";
	}
	elsif ($pktsreport){
		print OUTPUT_HANDLE "start log time          : instances: flow (ip_src.port->ip_dst.port[proto]): avg flow pkts/sec\n";
	}
	else {
		print OUTPUT_HANDLE "start log time          : instances: flow (ip_src.port->ip_dst.port[proto]): avg flow bits/sec\n";
	}
	print OUTPUT_HANDLE "end log time\n";  
	print OUTPUT_HANDLE "----------------------------------------------------------------------------------------\n";
	for ( $i = 0; $i < $asciitopn; $i++) {
		$flowid = $sortedipflows[$i];
		if ($flowid eq "") {
			close(OUTPUT_HANDLE);
			return;
		}
		$tstring = ctime($ipflow_time{$flowid}); 
		$tstring2 = ctime($ipflow_lasttime{$flowid}); 
		$hits = $ipflow_instances{$flowid}; 
		# convert to average bits per second
		# unless we are doing pkts/sec
		if ($pktsreport) {
			$bits = int((($ipflow{$flowid}) / 30)/$hits);
		}
		else {
			$bits = int((($ipflow{$flowid} * 8) / 30)/$hits);
		}
		($lhs_ip, $lhs_port, $rhs_ip, $rhs_port, $proto) = 
			lookup_flowid($flowid);
		$dnsname = gethostip($lhs_ip);
		if ($dnsname) {
			$lhs_ip = $dnsname; 
		}
		$dnsname = gethostip($rhs_ip);
		if ($dnsname) {
			$rhs_ip = $dnsname; 
		}
		if ( $icmpreport ) {
			my $imaj = icmp_major($lhs_port);
			my $imin = icmp_minor($lhs_port, $rhs_port);

			$lflow = $lhs_ip  .  "->" . $rhs_ip . ':' 
				. '[' . $imaj . "\/" .  $imin . ']'; 
			print OUTPUT_HANDLE "$tstring", ": $hits",
				": ", "$lflow", 
				":", "$bits", "\n";
		}
		else {
			# if no port (-1) then print without ports
			# else include ports
			if ($lhs_port == -1) {
				$lflow = $lhs_ip  . "->" . $rhs_ip . ':' . '[' . $proto . ']'; 
			}
			else {
				$lflow = $lhs_ip  . '.' .  $lhs_port . "->" . $rhs_ip . '.' . 
					$rhs_port . ':' . '[' . $proto . ']'; 
			}
			$af = get_appflag($ipflow_uaphw{$fid}, $ipflow_uappl{$fid},
				$ipflow_uapph{$fid});
			print OUTPUT_HANDLE "$tstring", ": $hits",
				": ", "$lflow", 
				":", "$bits", ":", "$af", "\n";
		}
		print OUTPUT_HANDLE "$tstring2\n";
	}
	close(OUTPUT_HANDLE);
}

#
# turn icmp major # into string representation
# See BSD /sys/netinet/ip_icmp.h
#
sub icmp_major {
	my $major = $_[0];

        if ($major == 0) {
                return("reply");
        }
        elsif ($major == 3) {
                return("unreach");
        }
        elsif ($major == 4) {
                return("srcqnch");
        }
        elsif ($major == 5) {
                return("redirect");
        }
        elsif ($major == 8) {
                return("echo");
        }
        elsif ($major == 9) {
                return("rtadvrt");
        }
        elsif ($major == 10) {
                return("rtsolicit");
        }
        elsif ($major == 11) {
                return("timxceed");
        }
        elsif ($major == 12) {
                return("paramprob");
        }
        elsif ($major == 13) {
                return("tstamp");
        }
        elsif ($major == 14) {
                return("tstampreply");
        }
        elsif ($major == 15) {
                return("ireq");
        }
        elsif ($major == 16) {
                return("ireply");
        }
        elsif ($major == 17) {
                return("maskreq");
        }
        return($major);
}

sub icmp_minor {
	my $major = $_[0];
	my $minor = $_[1];

        if ($major == 3) {
		if ($minor == 0) {
			return("net");
		}
		elsif ($minor == 1) {
			return("host");
		}
		elsif ($minor == 2) {
			return("proto");
		}
		elsif ($minor == 3) {
			return("port");
		}
		elsif ($minor == 4) {
			return("needfrag");
		}
		elsif ($minor == 5) {
			return("srcfail");
		}
		elsif ($minor == 6) {
			return("netunknown");
		}
		elsif ($minor == 7) {
			return("hostunknown");
		}
		elsif ($minor == 8) {
			return("isolated");
		}
		elsif ($minor == 9) {
			return("netprohibit");
		}
		elsif ($minor == 13) {
			return("adminprohibit");
		}
        }
        elsif ($major == 5) {
		if ($minor == 0) {
                	return("net");
		}
		elsif ($minor == 1) {
                	return("host");
		}
		elsif ($minor == 2) {
                	return("tosnet");
		}
		elsif ($minor == 3) {
                	return("toshost");
		}
        }
        elsif ($major == 11) {
		if ($minor == 0) {
                	return("intransit");
		}
		elsif ($minor == 1) {
                	return("reass");
		}
        }
        return($minor);
}

sub lookup_flowid {
	$fid = $_[0];
	my @words;
	my @tmp2;
	my $lhs_ip; 
	my $lhs_port = -1; 
	my $rhs_ip; 
	my $rhs_port = -1; 
	my $proto;

	# note we have 2 possible formats, with ports/without ports:
	# 1. 1.1.1.1.1-2.2.2.2.2[tcp]
	# 2. 1.1.1.1-2.2.2.2[255]
	my @two = split(/\-/, $fid);	    # give us lhs rhs
	my @lhs = split(/\./, $two[0]);     # break lhs up into ints
	# break rhs up via next proto first
	@tmp2 = split(/\[/, $two[1]);	    # tmp1 has next proto
 	$proto = $tmp2[1]; 
	$proto =~ s/\]//;
	my @rhs = split(/\./, $tmp2[0]);     # break rhs up into ints 
	# determine type of format
	my $count = scalar @lhs;	    # either 4 or 5
	$lhs_ip = $lhs[0] . '.' . $lhs[1] . '.'  .
				$lhs[2] . '.' . $lhs[3];
	$rhs_ip = $rhs[0] . '.' . $rhs[1] . '.'  .
			$rhs[2] . '.' . $rhs[3];
	if ($count == 4) {
		$lhs_port = -1;
		$rhs_port = -1;
	}
	elsif ($count == 5) {
		$lhs_port = $lhs[4];
		$rhs_port = $rhs[4];
	}

# ACK
	if ($debug) {
		print "$lhs_ip\n";
		print "$lhs_port\n";
		print "$rhs_ip\n";
		print "$rhs_port\n";
		print "$proto\n\n";
	}
	return($lhs_ip, $lhs_port, $rhs_ip, $rhs_port, $proto);
}

# getTime - parse time stamp and return in unix epoch seconds
#
# large assumption: log file is same year as now,
# although this probably won't hurt much at year wrap time.
# (as the year isn't usually in the log file anyway).
# the only inputs we are paying attention to are month/day/hr/min/sec.
#
sub getTime()
{
	my $t1 = $_[0];  # month
	my $t2 = $_[1];  # day
	my $t3 = $_[2];  # hour:min:sec
	my $time;
	my $ltime;
        #%daylist = ( 0, 'Sun', 1, 'Mon', 2, 'Tue', 3, 'Wed', 
	# 	4, 'Thu', 5, 'Fri', 6, 'Sat');

	# use perl core routine for this
	# get now and simply patch 
	($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = 
			CORE::localtime(time);

	#print "sec ", $sec, "\n";
	#print "min ", $min, "\n";
	#print "hour ", $hour, "\n";
	#print "mday ", $mday, "\n";
	#print "mon ", $mon, "\n";
	#print "year ", $year, "\n";

        @hours = split(/:/, $t3);
	$hour = $hours[0];
	$min = $hours[1];
	$sec = $hours[2];
	$mday = $t2;
	$mon = $month{$t1};

	$time = timelocal($sec,$min,$hour,$mday,$mon,$year);

	$ltime = time;
	#printf("ltime: %s\n", ctime($ltime));
	#printf("time: %s\n", ctime($time));

	return($time);
}

# return dns from ip within timeout period of 5 seconds
#
sub gethostip {
	my $ipaddr = $_[0];
	my $dnsname;

	if ($dnscache{$ipaddr}) {
		if ($debug) {
 			print "cached $ipaddr: $dnscache{$ipaddr}\n";
		}
		return ($dnscache{$ipaddr});
	}


	if ($debug) {
		print "gethostip: lookup of $ipaddr\n";
	}
  	eval {
		$SIG{ALRM} = \&sigalrm; 
 		alarm(5);
		eval {
			$dnsname = gethostbyaddr(inet_aton($ipaddr), 2);
		};
		alarm(0);
 	};
	alarm(0);
#print "ip: $ipaddr dns: $dnsname\n";
	if ($dnsname) {
		$dnscache{$ipaddr} = $dnsname;
		if ($debug) {
			print "gethostip: found $dnsname\n";
		}
	}
	else {
		$dnscache{$ipaddr} = $ipaddr;
		if ($debug) {
			print "gethostip: did not find $ipaddr\n";
		}
	}
	return($dnsname);
}

sub sigalrm {
	#alarm(0);
	#print "ALARM TIMEOUT\n";
}

# always steal this "module" (bad form) from omupdate.pl
# parse syn_list app flags
sub get_appflag 
{
	my $appflags = $_[0];
	my $apptagsLower = $_[1];
	my $apptagsUpper = $_[2];
	my $emulehits = 1;

	my $af = parseAppFlags(hex($appflags), $emulehits);
	$af = parseLowerAppTags(hex($apptagsLower),$af);
	$af = parseUpperAppTags(hex($apptagsUpper),$af);
	return($af);
}

# ignoring emulehit count for now
sub parseAppFlags
{
	$appflags = $_[0];
	$emulehits = $_[1];
	#$appflags = hex($_[0]);
	my $string="";

	if ($appflags & 0x0f) {
		$string = "I";		#/* irc of some sort */
	}
	if ($appflags & 0x10) {
		$string .= "B";		#/* bittorrent */
	}
	if ($appflags & 0x20) {
		$string .= "G";		#/* GNUTELLA */
	}
	if ($appflags & 0x40) {
		$string .= "K";		#/* KAZAA */
	}
	if ($appflags & 0x80 && ($emulehits >= $emulecount)) {
		$string .= "e";		#/* edonkey */
	}
 	if ($appflags & 0x100) {
 		$string .= "P";		#/* honeypot dst */
 	}
	if ($appflags & 0x200) {
		$string .= "M";		#/* Morpheus */
	}
	if ($appflags & 0x400) {
		$string .= "H";		#/* web server based on tcp src port */
	}
	if ($appflags & 0x800) {
		$string .= "E";		#/* email server based on tcp dst port */
	}
	return($string);
}

sub parseLowerAppTags 
{
	$apptag = $_[0];
	$string = $_[1];

	if ($apptag & 0x1 ) { $string .= "a"; }
	if ($apptag & 0x2 ) { $string .= "b"; }
	if ($apptag & 0x4 ) { $string .= "c"; }
	if ($apptag & 0x8 ) { $string .= "d"; }
	if ($apptag & 0x10 ) { $string .= "e"; }
	if ($apptag & 0x20 ) { $string .= "f"; }
	if ($apptag & 0x40 ) { $string .= "g"; }
	if ($apptag & 0x80 ) { $string .= "h"; }
	if ($apptag & 0x100 ) { $string .= "i"; }
	if ($apptag & 0x200 ) { $string .= "j"; }
	if ($apptag & 0x400 ) { $string .= "k"; }
	if ($apptag & 0x800 ) { $string .= "l"; }
	if ($apptag & 0x1000 ) { $string .= "m"; }
	if ($apptag & 0x2000 ) { $string .= "n"; }
	if ($apptag & 0x4000 ) { $string .= "o"; }
	if ($apptag & 0x8000 ) { $string .= "p"; }
	if ($apptag & 0x10000 ) { $string .= "q"; }
	if ($apptag & 0x20000 ) { $string .= "r"; }
	if ($apptag & 0x40000 ) { $string .= "s"; }
	if ($apptag & 0x80000 ) { $string .= "t"; }
	if ($apptag & 0x100000 ) { $string .= "u"; }
	if ($apptag & 0x200000 ) { $string .= "v"; }
	if ($apptag & 0x400000 ) { $string .= "w"; }
	if ($apptag & 0x800000 ) { $string .= "x"; }
	if ($apptag & 0x1000000 ) { $string .= "y"; }
	if ($apptag & 0x2000000 ) { $string .= "z"; }

	return($string);
}

sub parseUpperAppTags 
{
	$apptag = $_[0];
	$string=  $_[1];

	if ($apptag & 0x1 ) { $string .= "A"; }
	if ($apptag & 0x2 ) { $string .= "B"; }
	if ($apptag & 0x4 ) { $string .= "C"; }
	if ($apptag & 0x8 ) { $string .= "D"; }
	if ($apptag & 0x10 ) { $string .= "E"; }
	if ($apptag & 0x20 ) { $string .= "F"; }
	if ($apptag & 0x40 ) { $string .= "G"; }
	if ($apptag & 0x80 ) { $string .= "H"; }
	if ($apptag & 0x100 ) { $string .= "I"; }
	if ($apptag & 0x200 ) { $string .= "J"; }
	if ($apptag & 0x400 ) { $string .= "K"; }
	if ($apptag & 0x800 ) { $string .= "L"; }
	if ($apptag & 0x1000 ) { $string .= "M"; }
	if ($apptag & 0x2000 ) { $string .= "N"; }
	if ($apptag & 0x4000 ) { $string .= "O"; }
	if ($apptag & 0x8000 ) { $string .= "P"; }
	if ($apptag & 0x10000 ) { $string .= "Q"; }
	if ($apptag & 0x20000 ) { $string .= "R"; }
	if ($apptag & 0x40000 ) { $string .= "S"; }
	if ($apptag & 0x80000 ) { $string .= "T"; }
	if ($apptag & 0x100000 ) { $string .= "U"; }
	if ($apptag & 0x200000 ) { $string .= "V"; }
	if ($apptag & 0x400000 ) { $string .= "W"; }
	if ($apptag & 0x800000 ) { $string .= "X"; }
	if ($apptag & 0x1000000 ) { $string .= "Y"; }
	if ($apptag & 0x2000000 ) { $string .= "Z"; }

	return($string);
}
